# include "vfieldfunc.h"

/* Lorenz Vector Field.

 Input:  (t, Y, p)
    t -- scalar (ignored)
    Y -- phase-space vector [x; y; z]
    p -- parameter vector [s; r; b]

 Output: 3-vector f = [x_dot; y_dot; z_dot] where

   x_dot = -s*x + s*y
   y_dot =  r*x -   y  - x*z
   z_dot =  y*x        - b*z
*/

void vfieldfunc(unsigned n,  double t, double* y, double* params, double* f)
{

  f[0] = params[0]*(y[1] - y[0]);
  f[1] = -y[0]*y[2] + params[1]*y[0] - y[1];
  f[2] = y[0]*y[1] - params[2]*y[2];
} 

/* when zdot = 0 */
void eventfunc (unsigned ndims, double t, double *y, double *params, 
					   int ievent, double &value, int &isterminal, int &direction)
{
	switch (ievent)
	{
	case 0:
		value = y[0]*y[1] - params[2]*y[2];
		isterminal = 0;
		direction = 1;
		break;
	}
}

